
from dynaconf import Dynaconf

settings = Dynaconf(
    envvar_prefix="JENKINS",
    settings_files=['~/.jenkinsctl.yaml', '~/.jenkinsctl.secrets.yaml'],
)

# `envvar_prefix` = export envvars with `export DYNACONF_FOO=bar`.
# `settings_files` = Load these files in the order.



import yaml

from .commons import use_vprint


def handle_get_config(args):
    vprint = use_vprint(args.verbose)
    vprint(f"Passed args : {vars(args)}")

    client = args.client()

    job_name = args.job_name

    build = get_build(client, vprint, job_name, args.build_no)

    params = get_params(build)

    print(to_yaml(job_name, params))


def get_build(client, vprint, job_name, build_no):
    job = client.get_job(job_name)

    final_build_no = get_last_build_no(job) if build_no is None else build_no
    vprint(f"final build no {final_build_no}")

    build = job.get(final_build_no)

    return build


def get_last_build_no(job):
    return job.get_last_build().number


def get_params(build):
    return dict([(param.name, param.value) for param in build.get_parameters()])


def to_yaml(job_name, params):
    return yaml.dump({"job": job_name, "params": params})



#!/usr/bin/env python3
# PYTHON_ARGCOMPLETE_OK

import argparse
import argcomplete
from .jbuild import handle_build_command
from api4jenkins import Jenkins
from .config import settings
from .jget_config import handle_get_config


server_url = settings.server_url
username = settings.username
api_key = settings.api_key


def get_client():
    return Jenkins(server_url, auth=(username, api_key))


def get_args():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(title="Subcommand", dest="subcommand")

    add_build_subparser(subparsers)
    add_get_config_subparser(subparsers)

    argcomplete.autocomplete(parser)
    args = parser.parse_args()
    args.func(args)


def add_build_subparser(subparsers):
    subparser = subparsers.add_parser("build", help="run new build")
    subparser.set_defaults(func=handle_build_command, client=get_client)
    subparser.add_argument(
        "-f", "--file", type=argparse.FileType("r"), help="Yaml configuration file"
    )
    subparser.add_argument("-v", "--verbose", action="store_true")
    subparser.add_argument("-s", "--suppress-logs", action="store_true")
    subparser.add_argument("--param", action="append", default=[])


def add_get_config_subparser(subparsers):
    subparser = subparsers.add_parser("config", help="get config of a build")
    subparser.set_defaults(func=handle_get_config, client=get_client)
    subparser.add_argument("job_name")
    subparser.add_argument("build_no", type=int, nargs="?")
    subparser.add_argument("-v", "--verbose", action="store_true")


if __name__ == '__main__':
    get_args()



#!/usr/bin/env python3
# PYTHON_ARGCOMPLETE_OK

import time
import sys
import yaml
from threading import Thread

from .commons import use_vprint


def handle_build_command(args):
    vprint = use_vprint(args.verbose)
    vprint(f"Passed args : {vars(args)}")

    conf = get_conf(args, vprint)
    vprint("Final config: ", conf)

    create_build(args.client, conf, args.suppress_logs, vprint)


def get_config_from_yaml(file):
    config_data = None
    try:
        with open(file.name, "r") as config_file:
            config_data = yaml.safe_load(config_file)  # Use safe_load for YAML
    except FileNotFoundError:
        print(f"Error: Configuration file '{file}' not found.")
        sys.exit()
    except yaml.YAMLError as e:
        print(f"Error: Invalid YAML format in '{file}': {e}")
        sys.exit()

    return config_data


def get_conf(args, vprint):
    config = get_config_from_yaml(args.file)
    vprint(f"Config from file : {config}")
    override_params(args, config)
    return config


def override_params(args, file_config):
    for param in args.param:
        name, value = param.split('=')
        file_config['params'][name] = value


def get_build(queued_item):
    build = queued_item.get_build()
    while build == None:
        time.sleep(2)
        build = queued_item.get_build()
    return build


def get_build_no(queued_item):
    return queued_item.api_json()["executable"]["number"]


def print_build_log(build):
    for line in build.progressive_output():
        print(line)


def approve_pending_input(build):
    if not hasattr(build, 'get_pending_input'):
        return

    while not build.get_pending_input() and build.building:
        time.sleep(1)

    if build.building:
        build.get_pending_input().submit()


def create_build(client, conf, suppress_logs: bool, vprint=print):

    client = client()
    vprint(f"client version: {client.version}")

    job = client.get_job(conf["job"])
    vprint(f"job : {job}")

    queued_item = job.build(**conf["params"])
    vprint(f"queued : {queued_item}")

    build = get_build(queued_item)
    vprint(f"build : {build}")

    build_number = get_build_no(queued_item)

    if not suppress_logs:
        print(f"STARTED... build number : {build_number}")

    approve_pending_input_thread = Thread(
        target=approve_pending_input, args=(build,))
    approve_pending_input_thread.start()

    if not suppress_logs:
        log_thread = Thread(target=print_build_log, args=(build,))
        log_thread.start()
        log_thread.join()

    approve_pending_input_thread.join()

    if build.result == "FAILURE":
        print(f"FAILED... build number : {build_number}")
        sys.exit(1)

    if not suppress_logs:
        print(f"FINISHED... build number : {build_number}")
    else:
        print(build_number)
